require 'spec_helper'

describe MySQLEncryptor do

  before(:each) do
    @cut = MySQLEncryptor.instance
  end

  context '#encrypt' do

    it "should return nil if the value passed in the options is nil" do
      @cut.should_receive(:mysql_encrypt).
        with(nil, 'the_key_value').
        and_return(nil)

      expect(@cut.encrypt(value: nil, key: 'the_key_value')).
        to be_nil
    end

    it "should convert the value to a string before having the mysql_encrypt method encrypt it" do
      @cut.should_receive(:mysql_encrypt).
        with("12", 'the_key_value').
        and_return("encrypted_12")

      expect(@cut.encrypt(value: 12, key: 'the_key_value')).
        to eq("encrypted_12")
    end

    it "should convert the value to a date string before encryption when the type is specified as a 'date'" do
      date = Date.today
      date_str = date.strftime("%Y%m%d")
      @cut.should_receive(:encrypt_date).
        with(date).and_return(date_str)
      @cut.should_receive(:mysql_encrypt).
        with(date_str, 'the_key_value').
        and_return("encrypted_date")

      expect(@cut.encrypt(value: date, key: 'the_key_value', type: 'date')).
        to eq("encrypted_date")
    end

  end

  context '#decrypt' do

    it "should return nil if mysql_decrypt returns a nil" do
      @cut.should_receive(:mysql_decrypt).
        with(nil, 'the_key').
        and_return(nil)

      expect(@cut.decrypt(value: nil, key: 'the_key')).
        to be_nil
    end

    it "should force the encoding of the decrypted string if the type is 'text'" do
      decrypted_value = 'decrypted_value'
      decrypted_value.should_receive(:force_encoding).
        with('utf-8').
        and_return(decrypted_value)

      @cut.should_receive(:mysql_decrypt).
        with('encrypted_value', 'the_key').
        and_return(decrypted_value)

      expect(@cut.decrypt(value: 'encrypted_value', key: 'the_key', type: 'text')).
        to eq(decrypted_value)
    end

    it "should return a Date object if the type of the encrypted value is 'date'" do
      date = Date.today
      date_str = date.strftime("%Y%m%d")

      @cut.should_receive(:mysql_decrypt).
        with('encrypted_date', 'the_key').
        and_return(date_str)

      expect(@cut.decrypt(value: 'encrypted_date', key: 'the_key', type: 'date')).
        to eq(date)
    end

    it "should return a DateTime object if the type of the encrypted value is 'datetime'" do
      date = DateTime.now.utc
      date_str = date.strftime("%Y-%m-%d %H:%M:%S")

      @cut.should_receive(:mysql_decrypt).
        with('encrypted_datetime', 'the_key').
        and_return(date_str)

      expect(@cut.decrypt(value: 'encrypted_datetime', key: 'the_key', type: 'datetime').strftime("%Y-%m-%d %H:%M:%S")).
        to eq(date_str)
    end

    it "should return the raw-unprocessed encrypted data if the value is binary" do
      @cut.should_receive(:mysql_decrypt).
        with('encrypted_binary', 'the_key').
        and_return("unencrypted_binary")

      expect(@cut.decrypt(value: 'encrypted_binary', key: 'the_key', type: 'binary')).
        to eq("unencrypted_binary")
    end

    it "should raise a runtime exception if the specified data type is not recognized" do
      @cut.should_receive(:mysql_decrypt).
        with('value', 'the_key').
        and_return("unencrypted_binary")
      expect { @cut.decrypt(value: 'value', key: 'the_key', type: 'bad_type') }.
        to raise_error(RuntimeError)
    end

  end

  context '#encrypt_date' do

    it "should return nil if a nil value is specified" do
      expect(@cut.encrypt_date(nil)).
        to be_nil
    end

    it "should parse specified value as a date if a string is specified, then format it appropriately" do
      expect(@cut.encrypt_date("2013/09/08")).
        to eq("20130908")
    end

    it "should use the date specified and format it appropriately" do
      date = Date.today
      date_str = date.strftime("%Y%m%d")
      expect(@cut.encrypt_date(date)).
        to eq(date_str)
    end

  end
end
